@@ -12,7 +12,7 @@ module Listen
 
       # TODO: use children(with_directory: false)
       path = dir + rel_path
-      current = Set.new(path.children)
+      current = Set.new(_children(path))
 
       Listen::Logger.debug do
         format('%s: %s(%s): %s -> %s',
@@ -20,10 +20,16 @@ module Listen
                rel_path, options.inspect, previous.inspect, current.inspect)
       end
 
-      current.each do |full_path|
-        type = detect_type(full_path)
-        item_rel_path = full_path.relative_path_from(dir).to_s
-        _change(snapshot, type, item_rel_path, options)
+      begin
+        current.each do |full_path|
+          type = ::File.lstat(full_path.to_s).directory? ? :dir : :file
+          item_rel_path = full_path.relative_path_from(dir).to_s
+          _change(snapshot, type, item_rel_path, options)
+        end
+      rescue Errno::ENOENT
+        # The directory changed meanwhile, so rescan it
+        current = Set.new(_children(path))
+        retry
       end
 
       # TODO: this is not tested properly
@@ -67,14 +73,15 @@ module Listen
       snapshot.invalidate(type, path, opts)
     end
 
-    def self.detect_type(full_path)
-      # TODO: should probably check record first
-      stat = ::File.lstat(full_path.to_s)
-      stat.directory? ? :dir : :file
-    rescue Errno::ENOENT
-      # TODO: ok, it should really check the record here
-      # report as dir for scanning
-      :dir
+    def self._children(path)
+      return path.children unless RUBY_ENGINE == 'jruby'
+
+      # JRuby inconsistency workaround, see:
+      # https://github.com/jruby/jruby/issues/3840
+      exists = path.exist?
+      directory = path.directory?
+      return path.children unless exists && !directory
+      raise Errno::ENOTDIR, path.to_s
     end
   end
 end
