@@ -6,14 +6,16 @@ module Listen
       # file: "/home/me/watched_dir", "app/models", "foo.rb"
       # dir, "/home/me/watched_dir", "."
       def initialize(root, relative, name = nil)
-        @root, @relative, @name = root, relative, name
+        @root = root
+        @relative = relative
+        @name = name
       end
 
       attr_reader :root, :relative, :name
 
       def children
         child_relative = _join
-        (Dir.entries(sys_path) - %w(. ..)).map do |name|
+        (_entries(sys_path) - %w(. ..)).map do |name|
           Entry.new(@root, child_relative, name)
         end
       end
@@ -46,6 +48,17 @@ module Listen
         args = [@relative, @name].compact
         args.empty? ? nil : ::File.join(*args)
       end
+
+      def _entries(dir)
+        return Dir.entries(dir) unless RUBY_ENGINE == 'jruby'
+
+        # JRuby inconsistency workaround, see:
+        # https://github.com/jruby/jruby/issues/3840
+        exists = ::File.exist?(dir)
+        directory = ::File.directory?(dir)
+        return Dir.entries(dir) unless exists && !directory
+        raise Errno::ENOTDIR, dir
+      end
     end
   end
 end
